Digraph = require('./digraph.js');

describe('Config Tests', () => {
  it('default', () => {
    digraph = new Digraph([])
    expect(digraph.edges()).toEqual([])
  });

  it('descendants no edges', () => {
    digraph = new Digraph([])
    expect(digraph.descendants(1)).toEqual(new Set([]))
  });
  
  it('descendants one edge', () => {
    digraph = new Digraph([[1,2]])
    expect(digraph.descendants(1)).toEqual(new Set([2]))
  });
  
  it('descendants two edges', () => {
    digraph = new Digraph([[1,2], [2,3]])
    expect(digraph.descendants(1)).toEqual(new Set([2,3]))
  });
  
  it('ancestors no edges', () => {
    digraph = new Digraph([])
    expect(digraph.ancestors(1)).toEqual(new Set([]))
  });
  
  it('ancestors one edge', () => {
    digraph = new Digraph([[1,2]])
    expect(digraph.ancestors(2)).toEqual(new Set([1]))
  });
  
  it('ancestors two edges', () => {
    digraph = new Digraph([[1,2], [2,3]])
    expect(digraph.ancestors(3)).toEqual(new Set([1,2]))
  });
  
  it('minima no edges one element', () => {
    digraph = new Digraph([])
    expect(digraph.minima([1])).toEqual(new Set([1]))
  });

  it('minima no edges two elements', () => {
    digraph = new Digraph([[]])
    expect(digraph.minima([1,2])).toEqual(new Set([1,2]))
  });

  it('minima one edge', () => {
    digraph = new Digraph([[1,2]])
    expect(digraph.minima([1,2])).toEqual(new Set([1]))
  });

  it('minima two minimas', () => {
    digraph = new Digraph([[1,2], [2,3], [4,5]])
    expect(digraph.minima([1,2,3,4,5])).toEqual(new Set([1,4]))
  });
  
  it('maxima no edges one element', () => {
    digraph = new Digraph([])
    expect(digraph.maxima([1])).toEqual(new Set([1]))
  });

  it('maxima no edges two elements', () => {
    digraph = new Digraph([[]])
    expect(digraph.maxima([1,2])).toEqual(new Set([1,2]))
  });

  it('maxima one edge', () => {
    digraph = new Digraph([[1,2]])
    expect(digraph.maxima([1,2])).toEqual(new Set([2]))
  });

  it('maxima two maximas', () => {
    digraph = new Digraph([[1,2], [2,3], [4,5]])
    expect(digraph.maxima([1,2,3,4,5])).toEqual(new Set([3,5]))
  });
})
