class Semantic {
  constructor(matcher, action) {
    this.matcher = matcher;
    this.action = action;
  }

  toString() {
    return `Semantic(${this.matcher}, ${this.action})`;
  }

  matches(objects, context) {
    return this.matcher({objects, context})
  }

  apply(objects, context, s) {
    const context_prime = Object.assign({}, context);
    this.action({objects, context: context_prime, s});
    return context_prime;
  }
}

class Semantics {
  constructor(semantics) {
    this.semantics = semantics
    this.logs = []
  };

  applyToContext(objects, context) {
    let context_prime = {}
    if (! (context instanceof Array || context instanceof Object) ){
      return context;
    }

    for ( const key in context ) {
      context_prime[key] = this.apply(objects, context[key])
    }
    const s = (c) => this.applyToContext(objects, c)
    for (const isemantic in this.semantics) {
      const semantic = this.semantics[isemantic]
      if (semantic.matches(objects, context)) {
        context_prime = semantic.apply(objects, context, s);
        this.logs.push(`Semantics: applied ${semantic.toString()}\n to\n ${JSON.stringify(context)}\n result is ${JSON.stringify(context_prime)}`)
        break
      }
    }
    return context_prime
  }

  applyToContexts(objects, contexts) {
    const contexts_prime = [];
    contexts.forEach( (context) => {
      contexts_prime.push(this.applyToContext(objects, context));
    });
    return contexts_prime;
  }

  apply(objects, value) {
    if ( Array.isArray(value) ) {
      return this.applyToContexts(objects, value)
    } else if ( value instanceof Object) {
      return this.applyToContext(objects, value)
    } else {
      return value
    }
  }
}

module.exports = { Semantic, Semantics };
